/*
             LUFA Library
     Copyright (C) Dean Camera, 2017.

  dean [at] fourwalledcubicle [dot] com
           www.lufa-lib.org
*/

/*
  Copyright 2017  Dean Camera (dean [at] fourwalledcubicle [dot] com)

  Permission to use, copy, modify, distribute, and sell this
  software and its documentation for any purpose is hereby granted
  without fee, provided that the above copyright notice appear in
  all copies and that both that the copyright notice and this
  permission notice and warranty disclaimer appear in supporting
  documentation, and that the name of the author not be used in
  advertising or publicity pertaining to distribution of the
  software without specific, written prior permission.

  The author disclaims all warranties with regard to this
  software, including all implied warranties of merchantability
  and fitness.  In no event shall the author be liable for any
  special, indirect or consequential damages or any damages
  whatsoever resulting from loss of use, data or profits, whether
  in an action of contract, negligence or other tortious action,
  arising out of or in connection with the use or performance of
  this software.
*/

/** \file
 *
 *  Main source file for the MassStorage demo. This file contains the main tasks of
 *  the demo and is responsible for the initial application hardware configuration.
 */

#include "MassStorage.h"

/** LUFA Mass Storage Class driver interface configuration and state information. This structure is
 *  passed to all Mass Storage Class driver functions, so that multiple instances of the same class
 *  within a device can be differentiated from one another.
 */
USB_ClassInfo_MS_Device_t Disk_MS_Interface =
	{
		.Config =
			{
				.InterfaceNumber           = INTERFACE_ID_MassStorage,
				.DataINEndpoint            =
					{
						.Address           = MASS_STORAGE_IN_EPADDR,
						.Size              = MASS_STORAGE_IO_EPSIZE,
						.Banks             = 1,
					},
				.DataOUTEndpoint           =
					{
						.Address           = MASS_STORAGE_OUT_EPADDR,
						.Size              = MASS_STORAGE_IO_EPSIZE,
						.Banks             = 1,
					},
				.TotalLUNs                 = TOTAL_LUNS,
			},
	};


/** Main program entry point. This routine contains the overall program flow, including initial
 *  setup of all components and the main program loop.
 */

#define RED_DDR      DDRC
#define RED_PORT     PORTC
#define RED_BIT      5

#define BLUE_DDR     DDRC
#define BLUE_PORT    PORTC
#define BLUE_BIT     6

#define GREEN_DDR    DDRB
#define GREEN_PORT   PORTB
#define GREEN_BIT    7

#define BTN1_DDR     DDRD
#define BTN1_PIN     PIND
#define BTN1_BIT     4

#define BTN2_DDR     DDRD
#define BTN2_PIN     PIND
#define BTN2_BIT     5

#define BTN3_DDR     DDRD
#define BTN3_PIN     PIND
#define BTN3_BIT     6

#include <avr/io.h>
#include <util/delay.h>

int fadeAmount = 5;
int brightness = 0;

const int animationDelay = 100; // temps avant que cela passe à la couleur suivante
int counter_3 = 0;
int STATE_BUTTON_1 = 0;
bool while_setup;
int counter_2 = 1;
int counter = 0;
const int numColors = 255;

void setColor (unsigned char red, unsigned char green, unsigned char blue) 
{        
    DDRB = 1 << DDB0;
    TCCR0A = 1 << COM0A1 | 1 << WGM01 | 1 << WGM00;
    TCCR0B = 1 << CS00;
    
    RED_PORT = red;
    GREEN_PORT = green;
    BLUE_PORT = blue;
} 

long HSBtoRGB(float _hue, float _sat, float _brightness) {
    float red = 0.0;
    float green = 0.0;
    float blue = 0.0;
    
    if (_sat == 0.0) {
        red = _brightness;
        green = _brightness;
        blue = _brightness;
    } else {
        if (_hue == 360.0) {
            _hue = 0;
        }

        int slice = _hue / 60.0;
        float hue_frac = (_hue / 60.0) - slice;

        float aa = _brightness * (1.0 - _sat);
        float bb = _brightness * (1.0 - _sat * hue_frac);
        float cc = _brightness * (1.0 - _sat * (1.0 - hue_frac));
        
        switch(slice) {
            case 0:
                red = _brightness;
                green = cc;
                blue = aa;
                break;
            case 1:
                red = bb;
                green = _brightness;
                blue = aa;
                break;
            case 2:
                red = aa;
                green = _brightness;
                blue = cc;
                break;
            case 3:
                red = aa;
                green = bb;
                blue = _brightness;
                break;
            case 4:
                red = cc;
                green = aa;
                blue = _brightness;
                break;
            case 5:
                red = _brightness;
                green = aa;
                blue = bb;
                break;
            default:
                red = 0.0;
                green = 0.0;
                blue = 0.0;
                break;
        }
    }

    long ired = red * 255.0;
    long igreen = green * 255.0;
    long iblue = blue * 255.0;
    
    return ((long) ((ired << 16) | (igreen << 8) | (iblue)));
}

void programme(void) {
    if (counter == 0)
    {RED_PORT |= (0<<RED_BIT);
    GREEN_PORT &= ~(1<<GREEN_BIT);}
    else if (counter == 1)
    {BLUE_PORT |= (0<<BLUE_BIT);
    RED_PORT &= ~(1<<RED_BIT);}
    else if (counter == 2)
    {GREEN_PORT |= (0<<GREEN_BIT);
    BLUE_PORT &= ~(1<<BLUE_BIT);
    counter = 0;}
  /*//POUR LA LED
  float colorNumber = counter > numColors ? counter - numColors: counter;
  float saturation = 1;       // Entre 0 et 1 (0 = gris, 1 = saturé)
  float brightness = 0.5;     // Entre 0 et 1 (0 = noir, 1 max lumiere)
  float hue = (colorNumber / ((float)(numColors))) * 360;
  long color = HSBtoRGB(hue, saturation, brightness); 
  int red = color >> 16 & 255;
  int green = color >> 8 & 255;
  int blue = color & 255;
  setColor(red, green, blue);
  counter = (counter + 1) % (numColors * 2);
  
  //_delay_ms(animationDelay);
    
  //TEST BOUTONS
    (BTN1_PIN & ~(1<<BTN1_BIT))==0?0:1;
    (BTN2_PIN & ~(1<<BTN2_BIT))==0?0:1;
    (BTN3_PIN & ~(1<<BTN3_BIT))==0?0:1;
    
    //SI BOUTONS APUYES
  while (BTN1_BIT == 1 || BTN2_BIT == 1 || BTN3_BIT == 1) 
    {
    if (while_setup)
      {
        (BTN1_PIN & ~(1<<BTN1_BIT))==0?0:1;
        (BTN2_PIN & ~(1<<BTN2_BIT))==0?0:1;
        (BTN3_PIN & ~(1<<BTN3_BIT))==0?0:1;
        
        GREEN_PORT &= ~(1<<GREEN_BIT); //GREEN_PORT |= (1<<GREEN_BIT); 
        BLUE_PORT &= ~(1<<BLUE_BIT);
        
        while_setup = 0;
      }
    
    counter_3 += 1; 
    //_delay_ms(5);
    if (counter_3 == 20)
      {
      RED_PORT |= (1<<RED_BIT);
      }
    if (counter_3 == 40)
      {
      RED_PORT |= (0<<RED_BIT);
      counter_3 == 0;
      }
    }*/
}

int main(void)
{
	SetupHardware();

	LEDs_SetAllLEDs(LEDMASK_USB_NOTREADY);
	GlobalInterruptEnable();
    
    int counter=0;
    
	for (;;)
	{   if (counter % 100 == 0){
            programme();
        }
            MS_Device_USBTask(&Disk_MS_Interface);
            USB_USBTask();
            counter++;
	}
}

/** Configures the board hardware and chip peripherals for the demo's functionality. */
void SetupHardware(void)
{
#if (ARCH == ARCH_AVR8)
	/* Disable watchdog if enabled by bootloader/fuses */
	MCUSR &= ~(1 << WDRF);
	wdt_disable();

	/* Disable clock division */
	clock_prescale_set(clock_div_1);
#elif (ARCH == ARCH_XMEGA)
	/* Start the PLL to multiply the 2MHz RC oscillator to 32MHz and switch the CPU core to run from it */
	XMEGACLK_StartPLL(CLOCK_SRC_INT_RC2MHZ, 2000000, F_CPU);
	XMEGACLK_SetCPUClockSource(CLOCK_SRC_PLL);

	/* Start the 32MHz internal RC oscillator and start the DFLL to increase it to 48MHz using the USB SOF as a reference */
	XMEGACLK_StartInternalOscillator(CLOCK_SRC_INT_RC32MHZ);
	XMEGACLK_StartDFLL(CLOCK_SRC_INT_RC32MHZ, DFLL_REF_INT_USBSOF, F_USB);

	PMIC.CTRL = PMIC_LOLVLEN_bm | PMIC_MEDLVLEN_bm | PMIC_HILVLEN_bm;
#endif

	/* Hardware Initialization */
	LEDs_Init();
	USB_Init();
	DataflashManager_Initialisation();

	/* Check if the Dataflash is working, abort if not */
	if (!(DataflashManager_CheckDataflashOperation()))
	{
		LEDs_SetAllLEDs(LEDMASK_USB_ERROR);
		for(;;);
	}

	/* Clear Dataflash sector protections, if enabled */
	DataflashManager_ResetDataflashProtections();

	//ajout par moi	
    RED_DDR |= (1<<RED_BIT);
    BLUE_DDR |= (1<<BLUE_BIT);
    GREEN_DDR |= (1<<GREEN_BIT);
    BTN1_DDR &= ~(1<<BTN1_BIT);
    BTN2_DDR &= ~(1<<BTN2_BIT);
    BTN3_DDR &= ~(1<<BTN3_BIT);
	//ajout fin
}

/** Event handler for the library USB Connection event. */
void EVENT_USB_Device_Connect(void)
{
	LEDs_SetAllLEDs(LEDMASK_USB_ENUMERATING);
}

/** Event handler for the library USB Disconnection event. */
void EVENT_USB_Device_Disconnect(void)
{
	LEDs_SetAllLEDs(LEDMASK_USB_NOTREADY);
}

/** Event handler for the library USB Configuration Changed event. */
void EVENT_USB_Device_ConfigurationChanged(void)
{
	bool ConfigSuccess = true;

	ConfigSuccess &= MS_Device_ConfigureEndpoints(&Disk_MS_Interface);

	LEDs_SetAllLEDs(ConfigSuccess ? LEDMASK_USB_READY : LEDMASK_USB_ERROR);
}

/** Event handler for the library USB Control Request reception event. */
void EVENT_USB_Device_ControlRequest(void)
{
	MS_Device_ProcessControlRequest(&Disk_MS_Interface);
}

/** Mass Storage class driver callback function the reception of SCSI commands from the host, which must be processed.
 *
 *  \param[in] MSInterfaceInfo  Pointer to the Mass Storage class interface configuration structure being referenced
 */
bool CALLBACK_MS_Device_SCSICommandReceived(USB_ClassInfo_MS_Device_t* const MSInterfaceInfo)
{
	bool CommandSuccess;

	LEDs_SetAllLEDs(LEDMASK_USB_BUSY);
	CommandSuccess = SCSI_DecodeSCSICommand(MSInterfaceInfo);
	LEDs_SetAllLEDs(LEDMASK_USB_READY);

	return CommandSuccess;
}

